/*
 * *************************************************
 * Copyright (c) 2019 QMTH. All Rights Reserved.
 * Created by Deason on 2019-01-18 16:55:58.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.questions.api.bean;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.core.questions.base.question.enums.QuesStructType;
import cn.com.qmth.examcloud.core.questions.dao.entity.QuesOption;
import cn.com.qmth.examcloud.core.questions.dao.entity.Question;
import cn.com.qmth.examcloud.core.questions.service.bean.dto.PaperDetailExp;
import cn.com.qmth.examcloud.core.questions.service.bean.dto.PaperDetailUnitExp;
import cn.com.qmth.examcloud.core.questions.service.bean.dto.PaperExp;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

public class ParsePaper {
    private static final Logger log = LoggerFactory.getLogger(ParsePaper.class);
    private final static String SECTION_STYLE = "class=\"pDetail\"";//试卷大题样式
    private final static String QUESTION_STYLE = "class=\"pQues\"";//试题标题样式
    private final static String OPTION_STYLE = "class=\"pOpt\"";//试题答案项样式
    private final static String FORMAT_SECTION = "<p %s><b>%s、%s (本大题共%s小题，共%s分)</b></p>";//大题标题格式
    private final static String FORMAT_QUESTION = "<p %s>%s、%s (%s分)</p>";//试题标题格式
    private final static String FORMAT_SUB_QUESTION = "<p %s>(%s)、%s (%s分)</p>";//子试题标题格式
    private final static String FORMAT_OPTION = "<p %s>%s、%s</p>";//选项内容格式
    private final static String[] REGEX_FILL_1 = {"[#][#]", "____"};//正则关键字
    private final static String[] REGEX_FILL_2 = {"[_][#]", "_"};//正则关键字

    public static List<String> convertPaperHtml(PaperExp data) {
        if (data == null) {
            log.warn("Paper is null.");
            throw new StatusException("500", "试卷信息不存在！");
        }

        List<PaperDetailExp> details = data.getPaperDetails();
        if (details == null || details.size() == 0) {
            log.warn("Paper Detail is empty.");
            return Lists.newArrayList();
        }

        //封装试卷的HTML内容
        List<String> htmlList = new ArrayList<>();
        for (int x = 0; x < details.size(); x++) {
            PaperDetailExp detail = details.get(x);
            if (StringUtils.isNotBlank(detail.getTitle())) {
                htmlList.add(String.format(FORMAT_SECTION, SECTION_STYLE, detail.getCnNum(), detail.getName() + detail.getTitle(), detail.getUnitCount(), formatValue(detail.getScore())));
            } else {
                htmlList.add(String.format(FORMAT_SECTION, SECTION_STYLE, detail.getCnNum(), detail.getName(), detail.getUnitCount(), formatValue(detail.getScore())));
            }

            //封装大题的试题列表
            List<PaperDetailUnitExp> units = detail.getPaperDetailUnits();
            for (int y = 0; y < units.size(); y++) {
                PaperDetailUnitExp unit = units.get(y);
                Question question = unit.getQuestion();
                if (question == null) {
                    continue;
                }

                String qBody = question.getQuesBody();
                if (StringUtils.isEmpty(qBody)) {
                    log.warn("Question body is empty.");
                    qBody = " ";
                }

                //题干 替换特殊关键字
                qBody = replaceFill(question, qBody);
                htmlList.add(String.format(FORMAT_QUESTION, QUESTION_STYLE, unit.getNumber(), qBody, formatValue(question.getScore())));

                //客观题 选项列表
                List<QuesOption> options = question.getQuesOptions();
                if (options != null && options.size() > 0) {
                    for (QuesOption option : options) {
                        htmlList.add(String.format(FORMAT_OPTION, OPTION_STYLE, formatNum(option.getNumber()), replaceTagP(option.getOptionBody())));
                    }
                }

                if (question.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION) {
                    //套题的子题
                    List<Question> subQuestions = question.getSubQuestions();
                    if (subQuestions != null && subQuestions.size() > 0) {
                        for (int m = 0; m < subQuestions.size(); m++) {
                            Question subQues = subQuestions.get(m);

                            String subBody = subQues.getQuesBody();
                            if (StringUtils.isEmpty(subBody)) {
                                log.warn("SubQuestion body is empty.");
                                subBody = " ";
                            }

                            //题干 替换特殊关键字
                            subBody = replaceFill(subQues, subBody);
                            String subQuesNumber = subQues.getQuesParams().get("number");
                            htmlList.add(String.format(FORMAT_SUB_QUESTION, QUESTION_STYLE, subQuesNumber, subBody, formatValue(subQues.getScore())));

                            //客观题 选项列表
                            List<QuesOption> subOptions = subQues.getQuesOptions();
                            if (subOptions != null && subOptions.size() > 0) {
                                for (int n = 0; n < subOptions.size(); n++) {
                                    QuesOption subOpt = subOptions.get(n);
                                    htmlList.add(String.format(FORMAT_OPTION, OPTION_STYLE, formatNum(subOpt.getNumber()), replaceTagP(subOpt.getOptionBody())));
                                }
                            }
                        }
                    }
                }
            }
        }
        return htmlList;
    }

    public static List<String> convertAnswerHtml(PaperExp data) {
        if (data == null) {
            throw new StatusException("500", "试卷信息不存在！");
        }

        List<PaperDetailExp> details = data.getPaperDetails();
        if (details == null || details.size() == 0) {
            return Lists.newArrayList();
        }

        //封装试卷答案的HTML内容
        List<String> htmlList = new ArrayList<>();
        for (PaperDetailExp detail : details) {
            htmlList.add(String.format(FORMAT_SECTION, SECTION_STYLE, detail.getCnNum(), detail.getName(), detail.getUnitCount(), formatValue(detail.getScore())));

            List<PaperDetailUnitExp> units = detail.getPaperDetailUnits();
            for (PaperDetailUnitExp unit : units) {
                Question question = unit.getQuestion();
                if (question == null) {
                    continue;
                }

                if (question.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION) {
                    htmlList.add(String.format(FORMAT_QUESTION, QUESTION_STYLE, unit.getNumber(), "题", formatValue(question.getScore())));
                    List<Question> subQuestions = question.getSubQuestions();
                    if (subQuestions != null && subQuestions.size() > 0) {
                        for (Question subQues : subQuestions) {
                            String subQuesNumber = subQues.getQuesParams().get("number");
                            htmlList.add(String.format(FORMAT_SUB_QUESTION, QUESTION_STYLE, subQuesNumber, replaceFill(subQues, subQues.getQuesAnswer()), formatValue(subQues.getScore())));
                        }
                    }
                } else {
                    htmlList.add(String.format(FORMAT_QUESTION, QUESTION_STYLE, unit.getNumber(), replaceFill(question, question.getQuesAnswer()), formatValue(question.getScore())));
                }
            }
        }
        return htmlList;
    }

    public static String formatValue(Double score) {
        if (score == null) {
            return "";
        }
        DecimalFormat decimalFormat = new DecimalFormat("#.#");
        return decimalFormat.format(score);
    }

    public static String formatNum(String numStr) {
        int number = Integer.parseInt(numStr);
        char c = (char) (64 + number);
        return String.valueOf(c);
    }

    private static String replaceFill(Question question, String body) {
        if (StringUtils.isEmpty(body)) {
            return "";
        }

        body = replaceTagP(body);
        body = body.replaceAll(REGEX_FILL_1[0], REGEX_FILL_1[1]);

        //填空题情况
        if (question.getQuestionType() == QuesStructType.FILL_BLANK_QUESTION) {
            body = body.replaceAll(REGEX_FILL_2[0], REGEX_FILL_2[1]);
        }
        return body;
    }

    /**
     * 去掉"p"标签
     */
    public static String replaceTagP(String str) {
        if (str == null) {
            return null;
        }
        return str.replace("<p>", "")
                .replace("</p>", "");
    }

}